from datetime import datetime
from pydantic import BaseModel, model_validator

from xync_schema.enums import AdStatus, PmType
from xync_schema.models import Direction, Pmcur, Ex, Cred, Contragent


class _CurCoin(BaseModel):
    exid: int | str
    ticker: str
    rate: float | None = None


class CoinEpyd(_CurCoin):
    p2p: bool = True
    minimum: float | None = None


class CurEpyd(_CurCoin):
    rounding_scale: int | None = None
    minimum: int | None = None


class PmexBankPyd(BaseModel):
    id: int | None = None
    exid: str
    name: str


class PmPyd(BaseModel):
    id: int | None = None
    name: str
    identifier: str | None = None
    type_: PmType | None = None
    logo: str | None = None
    banks: list[PmexBankPyd] | None = None


# class PmcurPyd(BaseModel):
#     id: int | None = None
#     pm_id: int
#     cur_id: int


class FiatUpd(BaseModel):
    detail: str | None = None
    name: str | None = None
    amount: float | None = None
    target: int | None = None


class FiatNew(FiatUpd):
    cur_id: int
    pm_id: int
    detail: str
    amount: float = 0
    target: int | None = None


class CredPydIn(BaseModel):
    id: int | None = None
    exid: str
    ex: Ex | None = None
    ex_id: int | None = None
    pmcur_id: int | None = None
    pmcur: Pmcur | None = None
    detail: str
    name: str = ""

    class Config:
        arbitrary_types_allowed = True

    @classmethod
    @model_validator(mode="before")
    def check_at_least_one_field(cls, values):
        if (values.get("ex") or values.get("ex_id")) and (values.get("pmcur") or values.get("pmcur_id")):
            return values
        raise ValueError("ex(_id) and pmcur(_id) is required")


class FiatPydIn(BaseModel):
    # unq
    id: int = None
    cred_id: int | None = None
    cred: Cred | None = None
    # df
    amount: float
    target: float | None = None

    banks: list[str] = []

    class Config:
        arbitrary_types_allowed = True

    @classmethod
    @model_validator(mode="before")
    def check_at_least_one_field(cls, values):
        if values.get("cred") or values.get("cred_id"):
            return values
        raise ValueError("cred(_id) is required")

    def args(self) -> tuple[dict, dict]:
        unq: tuple[str, ...] = "id", "cred_id", "cred"
        df: tuple[str, ...] = "amount", "target"
        d = self.model_dump()
        return {k: getattr(self, k) for k in df if d.get(k)}, {k: getattr(self, k) for k in unq if d.get(k)}


class BaseAd(BaseModel):
    id: int | None = None
    price: float


class AdPydIn(BaseAd):
    min_fiat: float
    max_fiat: float | None = None
    detail: str | None = None
    auto_msg: str | None = None
    status: AdStatus = AdStatus.active
    maker_id: int | None = None
    maker: Contragent | None = None
    direction_id: int | None = None
    direction: Direction | None = None
    pms_: list | None = None
    fiats_: list | None = None

    class Config:
        arbitrary_types_allowed = True

    @classmethod
    @model_validator(mode="before")
    def check_at_least_one_field(cls, values):
        maker = values.get("maker") or values.get("maker_id")
        direction = values.get("direction") or values.get("direction_id")
        pms_or_fiats = values.get("pms_") or values.get("fiats_")
        if maker and direction and pms_or_fiats:
            return values
        raise ValueError("(pms or fiats) and maker(_id) and direction(_id) is required")


class OrderPyd(BaseModel):
    id: int
    amount: float
    status: str
    actions: dict | None = {}
    cred: Cred.pyd()
    is_sell: bool
    contragent: int | None = None
    created_at: datetime
    payed_at: datetime | None = None
    appealed_at: datetime | None = None
    confirmed_at: datetime | None = None
    msgs: int = 0
    topic: int


class UreadMsgs(BaseModel):
    order_id: int
    unread_cnt: int
